#ifndef FGGL_GFX_OGL_COMPAT_H
#define FGGL_GFX_OGL_COMPAT_H
/**
 * Legacy/Direct OpenGL calls.
 *
 * This shouldn't be exposed to the demo app, but the ECS we're using isn't smart enouph to allow us to
 * abstract this yet. It's next thing on the list, but this branch is about cleaning up OpenGL not about
 * extending our ECS.
 *
 * Should be removed when the engine has suitable abstractions in place.
 */

#include <fggl/gfx/ogl/shader.hpp>
#include <fggl/gfx/ogl/renderer.hpp>

#include <fggl/gfx/common.hpp>
#include <fggl/gfx/camera.hpp>
#include <fggl/ecs/ecs.hpp>

namespace fggl::gfx {

	//
	// fake module support - allows us to still RAII
	//
	struct ecsOpenGLModule {
		fggl::gfx::Graphics ogl;
		fggl::gfx::MeshRenderer renderer;
		fggl::gfx::ShaderCache cache;

		inline ecsOpenGLModule(Window& window, fggl::data::Storage& storage) : 
			ogl(window), renderer(), cache(storage) {
		}
	};
	using OglModule = std::shared_ptr<ecsOpenGLModule>;

	//
	// Loading related functions - should be handled in modules/data-driven
	//
	inline void loadPipeline(OglModule& mod, const std::string& name, bool hasGeom) {
		fggl::gfx::ShaderConfig config;
		config.name = name;
		config.vertex = name+"_vert.glsl";
		config.fragment = name+"_frag.glsl";
		if ( hasGeom ) {
			config.geometry = name+"_geom.glsl";
		}
		mod->cache.load(config);
	}

	inline void loadBasicPipeline(OglModule& mod, const std::string &name) {
		loadPipeline(mod, name, false);
	}

	//
	// fake module/callbacks - our ECS doesn't have module/callback support yet.
	// 
	inline void onStaticMeshAdded(ecs::ECS& ecs, ecs::entity_t entity, OglModule& mod) {
		auto meshData = ecs.getComponent<gfx::StaticMesh>(entity);
		auto pipeline = mod->cache.get(meshData->pipeline);

		auto glMesh = mod->renderer.upload(meshData->mesh);
		glMesh.pipeline = pipeline;

		ecs.addComponent<fggl::gfx::GlRenderToken>(entity, glMesh);
	}

	inline void renderMeshes(OglModule& mod, ecs::ECS& ecs, float dt) {
		// get the camera
		auto cameras = ecs.getEntityWith<fggl::gfx::Camera>();
		if ( cameras.empty() ) {
			return;
		}
		auto camera = cameras[0];

		// get the models
		auto renderables = ecs.getEntityWith<fggl::gfx::GlRenderToken>();
		for ( auto renderable : renderables ) {
			mod->renderer.render(ecs, camera, dt);
		}
	}

	inline void ecsInitGraphics(ecs::ECS& ecs) {
		ecs.registerComponent<fggl::gfx::StaticMesh>();
		ecs.registerComponent<fggl::gfx::Camera>();
	}

	inline std::shared_ptr<ecsOpenGLModule> ecsInitOpenGL(ecs::ECS& ecs, Window& window, data::Storage& storage) {
		ecsInitGraphics(ecs);

		auto mod = std::make_shared<ecsOpenGLModule>(window, storage);
		ecs.registerComponent<fggl::gfx::GlRenderToken>();
		return mod;
	}

}


#endif
